#include "precompile.h"

#include "entity.h"
#include "collider.h"
#include "room.h"
#include "building.h"
#include "human.h"

Entity::Entity()
{

}

Entity::~Entity()
{
    ClearColliders();
}

void Entity::Initialize()
{
    xtimer_attacher.xtimer = &room->xtimer;
}

ColliderComponent* Entity::GetBoxBound()
{
    CircleCollider* collider = new CircleCollider();
    collider->active = true;
    collider->owner = this;
    return collider;
}

bool Entity::TestCollision(Entity* b)
{
    for (auto& a_collider : colliders) {
        for (auto& b_collider : b->colliders) {
            if (a_collider->Intersect(b_collider)) {
                return true;
            }
        }
    }
    return false;
}

void Entity::ClearColliders()
{
    for (auto& itr : colliders) {
        ColliderComponent* collider = itr;
        DestoryCollider(collider);
    }
    colliders.clear();
}

void Entity::FindLocationWithTarget(Entity* target)
{
    ColliderComponent* a_collider = GetBoxBound();
    Vector2D old_pos = pos;
    Vector2D new_pos = pos;
    ColliderComponent* target_collider = target->GetBoxBound();
    {
        bool ret = a_collider->CalcSafePoint(target_collider, new_pos);
        if (!ret) {
            abort();
        }
    }
    Vector2D new_pos_dir = new_pos - old_pos;
    float distance = (new_pos - old_pos).Norm();
    new_pos_dir.Normalize();
    for (float i = distance; i < 10000000; i += 5.0f) {
        pos = old_pos + new_pos_dir * i;
        std::set<GridCell*> new_grid_list;
        room->grid_service.GetAllCellsByXy(pos.x, pos.y, new_grid_list);

        std::vector<Entity*> objects;
        for (auto& grid : new_grid_list) {
            for (Entity* entity : grid->entity_list) {
                switch (entity->entity_type) {
                case ET_Obstacle:
                    {
                        if (entity != this && TestCollision(entity)){
                            objects.push_back(entity);
                            break;
                        }
                    }
                    break;
                default:
                    {
                    }
                    break;
                }
            }
        }
        if (objects.empty()) {
            break;
        }
    }

    room->grid_service.RemoveFromGridList(grid_list, this);
    {
        grid_list.clear();
        switch (entity_type) {
        case ET_Player:
            {
                room->grid_service.AddHuman((Human*)this);
            }
            break;
        case ET_Bullet:
            {
                room->grid_service.AddBullet((Bullet*)this);
            }
            break;
        default:
            {
                room->grid_service.AddEntity(this);
            }
            break;
        }
    }

    DestoryCollider(target_collider);
    DestoryCollider(a_collider);
}

void Entity::BroadcastFullState()
{
    std::set<GridCell*> grid_list;
    room->grid_service.GetAllCells(grid_id, grid_list);
    for (auto& grid : grid_list) {
        for (Human* hum : grid->human_list) {
            hum->AddToNewObjects(this);
        }
    }
}

void Entity::BroadcastDeleteState()
{
    std::set<GridCell*> grid_list;
    room->grid_service.GetAllCells(grid_id, grid_list);
    for (auto& grid : grid_list) {
        for (Human* hum : grid->human_list) {
            hum->RemoveObjects(this);
        }
    }
}
