#include "precompile.h"

#include "jsondatamgr.h"
#include "app.h"

#include "framework/cpp/utils.h"

void JsonDataMgr::Init()
{
    std::string gameserver_cluster_json_file;
    std::string masterserver_cluster_json_file;
    if (!f8::IsOnlineEnv()) {
        if (App::Instance()->HasFlag(2)) {
            gameserver_cluster_json_file = a8::Format("/root/pub/%d/%d/conf_test/game%d/gameserver/node%d/"
                                                      "game%d.gameserver.cluster.json",
                                                      {
                                                       GAME_ID,
                                                       App::Instance()->instance_id,
                                                       GAME_ID,
                                                       App::Instance()->node_id,
                                                       GAME_ID
                                                      });
            masterserver_cluster_json_file = a8::Format("/root/pub/%d/%d/conf_test/game%d/gameserver/node%d/"
                                                        "game%d.masterserver.cluster.json",
                                                        {
                                                         GAME_ID,
                                                         App::Instance()->instance_id,
                                                         GAME_ID,
                                                         App::Instance()->node_id,
                                                         GAME_ID
                                                        });
        } else {
            gameserver_cluster_json_file = a8::Format("/var/data/conf_test/game%d/gameserver/node%d/"
                                                      "game%d.gameserver.cluster.json",
                                                      {GAME_ID, App::Instance()->node_id, GAME_ID});
            masterserver_cluster_json_file = a8::Format("/var/data/conf_test/game%d/gameserver/node%d/"
                                                        "game%d.masterserver.cluster.json",
                                                        {GAME_ID, App::Instance()->node_id, GAME_ID});
        }
    } else {
        gameserver_cluster_json_file = a8::Format("../config/node%d/game%d.gameserver.cluster.json",
                                                  {
                                                   App::Instance()->node_id,
                                                   GAME_ID
                                                  });
        masterserver_cluster_json_file = a8::Format("../config/node%d/game%d.masterserver.cluster.json",
                                                    {
                                                     App::Instance()->node_id,
                                                     GAME_ID
                                                    });
    }
    gameserver_cluster_json_.ReadFromFile(gameserver_cluster_json_file);
    masterserver_cluster_json_.ReadFromFile(masterserver_cluster_json_file);
    ip = GetConf()->At("ip")->AsXValue().GetString();
    listen_port = GetConf()->At("listen_port")->AsXValue();
}

void JsonDataMgr::UnInit()
{
}

std::shared_ptr<a8::XObject> JsonDataMgr::GetConf()
{
    if (App::Instance()->instance_id < 1 || App::Instance()->instance_id > gameserver_cluster_json_.Size()) {
        abort();
    }
    return gameserver_cluster_json_[App::Instance()->instance_id - 1];
}

std::shared_ptr<a8::XObject> JsonDataMgr::GetMasterServerClusterConf()
{
    return std::make_shared<a8::XObject>(masterserver_cluster_json_);
}
