#pragma once

#include "metadata.h"

class MetaDataLoader;
class MetaMgr : public a8::Singleton<MetaMgr>
{

 private:
    MetaMgr() {};
    friend class a8::Singleton<MetaMgr>;

 public:

    void Init();
    void UnInit();
    void Reload();

    int GetSysParamAsInt(const std::string& param_name, int def_val = 0);
    double GetSysParamAsFloat(const std::string& param_name, double def_val = 0.0f);
    std::string GetSysParamAsString(const std::string& param_name, const char* def_val = "");
    MetaData::Map* GetMap(int map_id);
    MetaData::MapThing* GetMapThing(int mapthing_id);
    MetaData::Player* GetPlayer(int id);
    MetaData::Equip* GetEquip(int id);
    MetaData::Equip* GetEquipBySlotId(int slot_id);
    MetaData::EquipUpgrade* GetEquipUpgrade(int equip_id, int equip_lv);
    MetaData::Building* GetBuilding(int building_id);
    MetaData::Drop* GetDrop(int drop_id);
    MetaData::SafeArea* GetSafeArea(int area_id);
    std::vector<MetaData::MapTplThing>* GetMapTplThing(std::string& map_name);
    std::list<MetaData::AirDrop>& GetAirDrops();
    MetaData::AirLine* RandAirLine();
    MetaData::Skill* GetSkill(int skill_id);
    MetaData::Dress* GetDress(int dress_id);
    float GetRankRewardParam(int rank);
    float GetKillRewardParam(int kill_num);
    std::vector<MetaData::Robot>* GetRobotList();
    MetaData::Robot* GetRobot(int robot_id);

    int gas_inactive_time = 10;
    int jump_time = 10;
    float K = 100.0f;
    float kill_param = 0.0f;
    float rank_param = 0.0f;
    int fighting_mode = 0;

 private:
    MetaDataLoader* loader_ = nullptr;
};
