#include "precompile.h"

#include "obstacle.h"
#include "metamgr.h"
#include "room.h"
#include "collider.h"
#include "building.h"
#include "human.h"
#include "app.h"

Obstacle::Obstacle():Entity()
{
    entity_type = ET_Obstacle;
    --App::Instance()->perf.entity_num[ET_Obstacle];
}

Obstacle::~Obstacle()
{
    --App::Instance()->perf.entity_num[ET_Obstacle];
}

void Obstacle::Initialize()
{
    Entity::Initialize();
    health = meta->i->hp();
    RecalcSelfCollider();
}

void Obstacle::RecalcSelfCollider()
{
    if (is_door) {
        //门
        if (!self_collider2_) {
            self_collider2_ = new AabbCollider();
            self_collider2_->owner = this;
            colliders.push_back(self_collider2_);
        }
        if (door_state == DoorStateClose) {
            self_collider2_->_min = Vector2D(0.0f - door_state0->width() / 2.0f,
                                             0.0f - door_state0->height() / 2.0f);
            self_collider2_->_max = Vector2D(door_state0->width() / 2.0f, door_state0->height() / 2.0f);
        } else {
            self_collider2_->_min = Vector2D(0.0f - door_state1->width() / 2.0f,
                                             0.0f - door_state1->height() / 2.0f);
            self_collider2_->_max = Vector2D(door_state1->width() / 2.0f, door_state1->height() / 2.0f);
        }
    } else if (meta->i->attack_type() != 2){
        switch (meta->i->type()) {
        case 1:
            {
                if (!self_collider_) {
                    self_collider_ = new CircleCollider();
                    self_collider_->owner = this;
                    colliders.push_back(self_collider_);
                }
                self_collider_->pos = Vector2D();
                self_collider_->rad = meta->i->height() / 2.0;
            }
            break;
        case 2:
            {
                if (!self_collider2_) {
                    self_collider2_ = new AabbCollider();
                    self_collider2_->owner = this;
                    colliders.push_back(self_collider2_);
                }
                self_collider2_->_min = Vector2D(meta->i->width() / -2.0f, meta->i->height() / -2.0f);
                self_collider2_->_max = Vector2D(meta->i->width() / 2.0f, meta->i->height() / 2.0f);
            }
            break;
        }
    }
}

void Obstacle::FillMFObjectPart(cs::MFObjectPart* part_data)
{
    part_data->set_object_type(ET_Obstacle);
    cs::MFObstaclePart* p = part_data->mutable_union_obj_2();
    p->set_obj_uniid(entity_uniid);
    pos.ToPB(p->mutable_pos());
    p->set_scale(1.0f);
}

void Obstacle::FillMFObjectFull(cs::MFObjectFull* full_data)
{
    full_data->set_object_type(ET_Obstacle);
    cs::MFObstacleFull* p = full_data->mutable_union_obj_2();
    p->set_obj_uniid(entity_uniid);
    pos.ToPB(p->mutable_pos());
    p->set_scale(1.0f);

    p->set_obstacle_id(meta->i->thing_id());
    p->set_health(health);
    p->set_dead(dead);
    p->set_dead_at_thisframe(dead ? dead_frameno <= room->frame_no : false);

    p->set_is_door(is_door);
    if (is_door) {
        p->set_door_id(door_id);
        p->set_door_open_times(door_open_times);
        p->set_door_old_state((int)door_state);
        p->set_door_new_state((int)door_state);
        p->set_door_house_uniid(door_house_uniid);
        p->set_door_house_id(building->meta->i->mapid());
        if (door_state == DoorStateClose) {
            p->set_door_width(door_state0->width());
            p->set_door_height(door_state0->height());
        } else {
            p->set_door_width(door_state1->width());
            p->set_door_height(door_state1->height());
        }
    }
}

ColliderComponent* Obstacle::GetBoxBound()
{
    if (self_collider_) {
        CircleCollider* collider = new CircleCollider();
        *collider = *self_collider_;
        return collider;
    }
    if (self_collider2_) {
        AabbCollider* collider = new AabbCollider();
        *collider = *self_collider2_;
        return collider;
    }
    return nullptr;
}

void Obstacle::Explosion(Bullet* bullet)
{
    float old_rad = self_collider_->rad;
    if (self_collider_) {
        self_collider_->rad = meta->i->damage_dia();
    }
    if (meta->i->damage_dia() > 0.01f &&
        meta->i->damage() > 0.01f) {
        std::set<Entity*> objects;
        std::set<GridCell*> grid_list;
        room->grid_service.GetAllCellsByXy(pos.x, pos.y, grid_list);
        for (auto& grid : grid_list) {
            for (Human* hum: grid->human_list) {
                if (TestCollision(hum)) {
                    objects.insert(hum);
                }
            }
            for (Entity* entity : grid->entity_list) {
                switch (entity->entity_type) {
                case ET_Obstacle:
                case ET_Building:
                    {
                        if (entity != this && TestCollision(entity)) {
                            objects.insert(entity);
                        }
                    }
                    break;
                default:
                    {
                    }
                    break;
                }
            }//end for
        }
        Vector2D bomb_pos = pos;
        room->frame_event.AddExplosion(bullet, meta->i->thing_id(), bomb_pos);
        for (auto& target : objects) {
            switch (target->entity_type) {
            case ET_Player:
                {
                    Human* hum = (Human*)target;
#if 1
                    if (!hum->dead) {
#else
                        if (!hum->dead && (hum->team_id == 0 || hum->team_id != player->team_id)) {
#endif
                            float dmg = meta->i->damage();
                            float def = hum->def + hum->buff.def_add;
                            float finaly_dmg = dmg * (1 - def/MetaMgr::Instance()->K);
                            hum->DecHP(finaly_dmg, VP_Mine, "地雷", VW_Mine);
                        }
                    }
                    break;
                    case ET_Obstacle:
                        {
                            Obstacle* obstacle = (Obstacle*)target;
                            if (!obstacle->dead && obstacle->meta->i->attack_type() == 1) {
                                float dmg = meta->i->damage();
                                float def = 0;
                                float finaly_dmg = dmg * (1 - def/MetaMgr::Instance()->K);

                                obstacle->health = std::max(0.0f, obstacle->health - finaly_dmg);
                                obstacle->dead = obstacle->health <= 0.01f;
                                obstacle->dead_frameno = room->frame_no;
                                if (obstacle->dead) {
                                    obstacle->ClearColliders();
                                    room->ScatterDrop(obstacle->pos, obstacle->meta->i->drop());
                                }
                                obstacle->BroadcastFullState();
                            }
                        }
                        break;
                        default:
                            break;

            }
        }
   }
   if (self_collider_) {
       self_collider_->rad = old_rad;
   }
}
