#include "precompile.h"

#include <a8/timer.h>
#include <a8/mutable_xobject.h>

#include "roommgr.h"
#include "room.h"
#include "cs_proto.pb.h"
#include "GGListener.h"
#include "player.h"
#include "playermgr.h"
#include "app.h"
#include "metamgr.h"
#include "jsondatamgr.h"
#include "playermgr.h"

#include "framework/cpp/httpclientpool.h"

void RoomMgr::Init()
{
    auto master_svr_cluster_conf = JsonDataMgr::Instance()->GetMasterServerClusterConf();
    for (int i = 0; i < master_svr_cluster_conf->Size(); ++i) {
        auto master_svr_conf = master_svr_cluster_conf->At(i);
        int instance_id = master_svr_conf->At("instance_id")->AsXValue();
        std::string remote_ip = master_svr_conf->At("ip")->AsXValue();
        int remote_port = master_svr_conf->At("listen_port")->AsXValue();

        a8::Timer::Instance()->AddDeadLineTimer(1000 + (i + 1),
                                                a8::XParams()
                                                .SetSender(instance_id)
                                                .SetParam1(remote_ip)
                                                .SetParam2(remote_port),
                                                [] (const a8::XParams& param)
                                                {
                                                    RoomMgr::Instance()->ReportServerState(
                                                                                           param.sender,
                                                                                           param.param1,
                                                                                           param.param2
                                                                                           );
                                                });
    }
}

void RoomMgr::UnInit()
{
    for (auto& pair : room_hash_) {
        pair.second->UnInit();
        delete pair.second;
    }
    for (auto& pair : over_room_hash_) {
        pair.second->UnInit();
        delete pair.second;
    }
}

void RoomMgr::Update(int delta_time)
{
    for (auto& pair : room_hash_) {
        pair.second->Update(delta_time);
    }
}

void RoomMgr::_CMJoin(f8::MsgHdr& hdr, const cs::CMJoin& msg)
{
    MetaData::Player* hum_meta = MetaMgr::Instance()->GetPlayer(40001);
    assert(hum_meta);
    if (!hum_meta) {
        abort();
    }
    if (App::Instance()->is_test_mode) {
        for (int i = 0; i < App::Instance()->test_param; ++i) {
            Room* room = GetJoinableRoom(msg.account_id());
            if (!room) {
                room = new Room();
                room->room_uuid = App::Instance()->NewUuid();
                assert(!GetRoomByUuid(room->room_uuid));
                if (GetRoomByUuid(room->room_uuid)) {
                    abort();
                }
                room->map_meta = MetaMgr::Instance()->GetMap(1001);
                room->Init();
                inactive_room_hash_[room->room_uuid] = room;
                room_hash_[room->room_uuid] = room;
            }
            if (i == 0) {
                {
            Player* hum = PlayerMgr::Instance()->CreatePlayerByCMJoin(hdr.socket_handle, msg);
            hum->meta = hum_meta;
            room->AddPlayer(hum);

                    cs::SMJoinedNotify notifymsg;
                    notifymsg.set_error_code(0);
                    room->FillSMJoinedNotify(hum, notifymsg);
                    GGListener::Instance()->SendToClient(hdr.socket_handle, hdr.seqid, notifymsg);
                }
                {
                    cs::SMMapInfo notifymsg;
                    notifymsg.set_map_id(room->map_meta->i->map_id());
                    room->FillSMMapInfo(notifymsg);
                    GGListener::Instance()->SendToClient(hdr.socket_handle, hdr.seqid, notifymsg);
                }
            }
        }
    } else {
        Room* room = GetJoinableRoom(msg.account_id());
        if (!room) {
            room = new Room();
            room->room_uuid = App::Instance()->NewUuid();
            assert(!GetRoomByUuid(room->room_uuid));
            if (GetRoomByUuid(room->room_uuid)) {
                abort();
            }
            room->map_meta = MetaMgr::Instance()->GetMap(1001);
            room->Init();
            inactive_room_hash_[room->room_uuid] = room;
            room_hash_[room->room_uuid] = room;
        }
        Player* hum = PlayerMgr::Instance()->CreatePlayerByCMJoin(hdr.socket_handle, msg);
        hum->meta = hum_meta;
        room->AddPlayer(hum);

        {
            cs::SMJoinedNotify notifymsg;
            notifymsg.set_error_code(0);
            room->FillSMJoinedNotify(hum, notifymsg);
            GGListener::Instance()->SendToClient(hdr.socket_handle, hdr.seqid, notifymsg);
        }
        {
            cs::SMMapInfo notifymsg;
            notifymsg.set_map_id(room->map_meta->i->map_id());
            room->FillSMMapInfo(notifymsg);
            GGListener::Instance()->SendToClient(hdr.socket_handle, hdr.seqid, notifymsg);
        }
    }
}

int RoomMgr::RoomNum()
{
    return room_hash_.size();
}

int RoomMgr::OverRoomNum()
{
    return over_room_hash_.size();
}

Room* RoomMgr::GetJoinableRoom(const std::string& account_id)
{
    for (auto& pair : inactive_room_hash_) {
        if (pair.second->CanJoin(account_id)) {
            return pair.second;
        }
    }
    return nullptr;
}

Room* RoomMgr::GetRoomByUuid(long long room_uuid)
{
    auto itr = room_hash_.find(room_uuid);
    return itr != room_hash_.end() ? itr->second : nullptr;
}

void RoomMgr::AddOverRoom(long long room_uuid)
{
    inactive_room_hash_.erase(room_uuid);
    a8::Timer::Instance()->AddDeadLineTimer(1000,
                                            a8::XParams()
                                            .SetSender(room_uuid),
                                            [] (const a8::XParams& param)
                                            {
                                                Room* room = RoomMgr::Instance()->GetRoomByUuid(param.sender);
                                                if (room) {
                                                    RoomMgr::Instance()->room_hash_.erase(room->room_uuid);
                                                    RoomMgr::Instance()->over_room_hash_[room->room_uuid] = room;
                                                }
                                            });
    a8::Timer::Instance()->AddDeadLineTimer(1000 + 100,
                                            a8::XParams()
                                            .SetSender(room_uuid),
                                            [] (const a8::XParams& param)
                                            {
                                                RoomMgr::Instance()->FreeOverRoom(param.sender);
                                            });
}

void RoomMgr::ActiveRoom(long long room_uuid)
{
    inactive_room_hash_.erase(room_uuid);
}

void RoomMgr::ReportServerState(int instance_id, const std::string& host, int port)
{
    std::string url = a8::Format("http://%s:%d/webapp/index.php?c=GS&a=report&",
                                 {
                                     host,
                                     port
                                 });
    a8::MutableXObject* url_params = a8::MutableXObject::NewObject();
    url_params->SetVal("node_id", App::Instance()->node_id);
    url_params->SetVal("instance_id", App::Instance()->instance_id);
    url_params->SetVal("ip", JsonDataMgr::Instance()->ip);
    url_params->SetVal("port", JsonDataMgr::Instance()->listen_port);
    url_params->SetVal("online_num", PlayerMgr::Instance()->OnlineNum());
    url_params->SetVal("room_num", RoomNum());
    url_params->SetVal("servicing", App::Instance()->servicing ? 1 : 0);
    f8::HttpClientPool::Instance()->HttpGet(a8::XParams()
                                            .SetSender(instance_id)
                                            .SetParam1(host)
                                            .SetParam2(port),
                                            [] (a8::XParams& param, a8::XObject& data)
                                            {
                                                a8::Timer::Instance()->AddDeadLineTimer(1000,
                                                                                        a8::XParams()
                                                                                        .SetSender(param.sender)
                                                                                        .SetParam1(param.param1)
                                                                                        .SetParam2(param.param2),
                                                                                        [] (const a8::XParams& param)
                                                                                        {
                                                                                            RoomMgr::Instance()->ReportServerState(
                                                                                                                                   param.sender,
                                                                                                                                   param.param1,
                                                                                                                                   param.param2
                                                                                                                                   );
                                                                                        });
                                            },
                                            [] (a8::XParams& param, const std::string& response)
                                            {
                                                a8::Timer::Instance()->AddDeadLineTimer(1000,
                                                                                        a8::XParams()
                                                                                        .SetSender(param.sender)
                                                                                        .SetParam1(param.param1)
                                                                                        .SetParam2(param.param2),
                                                                                        [] (const a8::XParams& param)
                                                                                        {
                                                                                            RoomMgr::Instance()->ReportServerState(
                                                                                                                                   param.sender,
                                                                                                                                   param.param1,
                                                                                                                                   param.param2
                                                                                                                                   );
                                                                                        });
                                            },
                                            url.c_str(),
                                            *url_params,
                                            rand()
                                            );
    delete url_params;
}

void RoomMgr::FreeOverRoom(long long room_uuid)
{
    if (App::Instance()->HasFlag(6)) {
        auto itr = over_room_hash_.find(room_uuid);
        if (itr != over_room_hash_.end()) {
            itr->second->UnInit();
            delete itr->second;
            over_room_hash_.erase(itr);
        }
    }
}
