#include "precompile.h"

#include <a8/mutable_xobject.h>

#include "gsmgr.h"
#include "GGListener.h"

void GSMgr::Init()
{

}

void GSMgr::UnInit()
{

}

void GSMgr::_SS_WSP_RequestTargetServer(f8::MsgHdr& hdr, const ss::SS_WSP_RequestTargetServer& msg)
{
    ss::SS_MS_ResponseTargetServer respmsg;
    respmsg.set_context_id(msg.context_id());
    GSNode* node = GetNodeByTeamId(msg.team_id());
    if (node) {
        respmsg.set_host(node->ip);
        respmsg.set_port(node->port);
    } else {
        node = AllocNode();
        if (node) {
            respmsg.set_host(node->ip);
            respmsg.set_port(node->port);
            team_hash_[msg.team_id()] = node;
        } else {
            respmsg.set_error_code(1);
            respmsg.set_error_msg("无法分配节点");
        }
    }
    GGListener::Instance()->SendMsg(hdr.socket_handle, respmsg);
}

void GSMgr::___GSReport(f8::JsonHttpRequest* request)
{
    std::string ip = request->request.Get("ip");
    int port = request->request.Get("port");
    int online_num = request->request.Get("online_num");
    int room_num = request->request.Get("room_num");
    int instance_id = request->request.Get("instance_id");
    int node_id = request->request.Get("node_id");
    bool servicing = request->request.Get("servicing");
    std::string key = ip + ":" + a8::XValue(port).GetString();

    auto itr = node_key_hash_.find(key);
    if (itr != node_key_hash_.end()) {
        if (itr->second.online_num != online_num ||
            itr->second.room_num != room_num ||
            itr->second.servicing != servicing
            ) {
            itr->second.online_num = online_num;
            itr->second.room_num = room_num;
            itr->second.servicing = servicing;
            RearrangeNode();
        }
    } else {
        GSNode gs;
        gs.node_id = node_id;
        gs.instance_id = instance_id;
        gs.online_num = online_num;
        gs.room_num = room_num;
        gs.ip = ip;
        gs.port = port;
        gs.servicing = servicing;
        node_key_hash_[key] = gs;
        node_sorted_list_.push_back(&node_key_hash_[key]);
        RearrangeNode();
    }

    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
}

void GSMgr::___GSList(f8::JsonHttpRequest* request)
{
    a8::MutableXObject* node_list = a8::MutableXObject::NewArray();

    for (auto& pair : node_key_hash_) {
        a8::MutableXObject* node = a8::MutableXObject::NewObject();
        node->SetVal("node_id", pair.second.node_id);
        node->SetVal("instance_id", pair.second.instance_id);
        node->SetVal("room_num", pair.second.room_num);
        node->SetVal("online_num", pair.second.online_num);
        node->SetVal("ip", pair.second.ip);
        node->SetVal("port", pair.second.port);
        node->SetVal("servicing", pair.second.servicing);
        node_list->Push(*node);
        delete node;
    }

    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
    request->resp_xobj->SetVal("node_list", *node_list);
    delete node_list;
}

GSNode* GSMgr::GetNodeByTeamId(const std::string& team_id)
{
    auto itr = team_hash_.find(team_id);
    return itr != team_hash_.end() ? itr->second : nullptr;
}

GSNode* GSMgr::AllocNode()
{
    if (node_sorted_list_.empty()) {
        return nullptr;
    }
    size_t rnd = std::min((size_t)5, node_sorted_list_.size());
    return node_sorted_list_[rand() % rnd];
}

void GSMgr::RearrangeNode()
{
    std::sort(node_sorted_list_.begin(), node_sorted_list_.end(),
              [] (const GSNode* a, const GSNode* b)
              {
                  return
                      !b->servicing ||
                      a->online_num < b->online_num ||
                      a->room_num < b->room_num ||
                      a->node_idx > b->node_idx;
              }
              );
}
