#include "precompile.h"

#include <string.h>

#include <a8/tcpclient.h>
#include <a8/tcpclient2.h>

#include "virtualclient.h"
#include "app.h"
#include "ss_proto.pb.h"
#include "cs_proto.pb.h"

const int PACK_MAX = 1024 * 64;

void VirtualClient::Init()
{
    recv_bufflen_ = 0;
    recv_buff_ = (char*) malloc(PACK_MAX * 2);

#if TCP_CLIENT2
    tcp_client_ = new a8::TcpClient2();
#else
    tcp_client_ = new a8::TcpClient();
#endif
    tcp_client_->remote_address = remote_ip;
    tcp_client_->remote_port = remote_port;
    tcp_client_->on_error = std::bind(&VirtualClient::on_error, this, std::placeholders::_1, std::placeholders::_2);
    tcp_client_->on_connect = std::bind(&VirtualClient::on_connect, this, std::placeholders::_1);
    tcp_client_->on_disconnect = std::bind(&VirtualClient::on_disconnect, this, std::placeholders::_1);
    tcp_client_->on_socketread = std::bind(&VirtualClient::on_socketread, this ,std::placeholders::_1, std::placeholders::_2, std::placeholders::_3);
    tcp_client_->Open();
}

void VirtualClient::UnInit()
{
    delete tcp_client_;
    tcp_client_ = nullptr;
}

void VirtualClient::Update()
{

}

void VirtualClient::SendJoin()
{
    cs::CMJoin msg;
    msg.set_server_id(2);
    msg.set_account_id(account);
    msg.set_baseskin(14001);
    SendMsg(msg);
}

void VirtualClient::SendMove()
{

}

#if TCP_CLIENT2
void VirtualClient::on_error(a8::TcpClient2* sender, int errorId)
#else
void VirtualClient::on_error(a8::TcpClient* sender, int errorId)
#endif
{
    a8::UdpLog::Instance()->Error("VirtualClient errorid=%d", {errorId});
}

#if TCP_CLIENT2
void VirtualClient::on_connect(a8::TcpClient2* sender)
#else
void VirtualClient::on_connect(a8::TcpClient* sender)
#endif
{
    recv_bufflen_ = 0;
    a8::UdpLog::Instance()->Info("target server connected", {});
    App::Instance()->AddIMMsg(IM_VirtualClientConnect,
                              a8::XParams()
                              .SetSender(instance_id)
                              );
}

#if TCP_CLIENT2
void VirtualClient::on_disconnect(a8::TcpClient2* sender)
#else
void VirtualClient::on_disconnect(a8::TcpClient* sender)
#endif
{
    recv_bufflen_ = 0;
    a8::UdpLog::Instance()->Info("target server %d disconnected after 10s later reconnect", {instance_id});
    App::Instance()->AddIMMsg(IM_VirtualClientDisconnect,
                              a8::XParams()
                              .SetSender(instance_id)
                              );
}

#if TCP_CLIENT2
void VirtualClient::on_socketread(a8::TcpClient2* sender, char* buf, unsigned int len)
#else
void VirtualClient::on_socketread(a8::TcpClient* sender, char* buf, unsigned int len)
#endif
{
    if (recv_bufflen_ + len > 2 * PACK_MAX) {
        recv_bufflen_ = 0;
        a8::UdpLog::Instance()->Debug("recvied target server too long message", {});
        return;
    } else {
        memmove(&recv_buff_[recv_bufflen_], buf, len);
        recv_bufflen_ += len;
    }

    bool warning = false;
    unsigned int offset = 0;
    while (recv_bufflen_ - offset >= sizeof(f8::PackHead)) {
        f8::PackHead* p = (f8::PackHead*) &recv_buff_[offset];
        if (p->magic_code == f8::MAGIC_CODE) {
            if (recv_bufflen_ - offset < sizeof(f8::PackHead) + p->packlen) {
                break;
            }
            App::Instance()->AddSocketMsg(SF_VirtualClient,
                                          instance_id,
                                          0,
                                          p->msgid,
                                          p->seqid,
                                          &recv_buff_[offset + sizeof(f8::PackHead)],
                                          p->packlen);
            offset += sizeof(f8::PackHead) + p->packlen;
        } else {
            warning = true;
            offset++;
            continue;
        }
    }

    if (warning) {
        a8::UdpLog::Instance()->Debug("recvied bad package", {});
    }
    if (offset > 0 && offset < recv_bufflen_) {
        memmove(recv_buff_, recv_buff_ + offset, recv_bufflen_ - offset);
    }
    recv_bufflen_ -= offset;
}

